@@ -1,6 +1,5 @@
 {-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE CPP #-}
 {- |  This module implements parsing of additional arguments embedded in a
       comment when stack is invoked as a script interpreter
 
@@ -58,15 +57,13 @@ import           Data.Attoparsec.Args
 import           Data.Attoparsec.Text ((<?>))
 import qualified Data.Attoparsec.Text as P
 import           Data.Char (isSpace)
-import           Data.Conduit
+import           Conduit
 import           Data.Conduit.Attoparsec
-import           Data.Conduit.Text (decodeUtf8)
 import           Data.List (intercalate)
 import           Data.Text (pack)
-import           Stack.Constants
 import           Stack.Prelude
 import           System.FilePath (takeExtension)
-import           System.IO (stderr, hPutStrLn)
+import           System.IO (hPutStrLn)
 
 -- | Parser to extract the stack command line embedded inside a comment
 -- after validating the placement and formatting rules for a valid
@@ -120,7 +117,7 @@ getInterpreterArgs file = do
     parseFile src =
          runConduit
        $ src
-      .| decodeUtf8
+      .| decodeUtf8C
       .| sinkParserEither (interpreterArgsParser isLiterate stackProgName)
 
     isLiterate = takeExtension file == ".lhs"
@@ -147,11 +144,7 @@ getInterpreterArgs file = do
 
     decodeError e =
       case e of
-#if MIN_VERSION_conduit_extra(1,2,0)
         ParseError ctxs _ (Position line col _) ->
-#else
-        ParseError ctxs _ (Position line col) ->
-#endif
           if null ctxs
           then "Parse error"
           else ("Expecting " ++ intercalate " or " ctxs)
