@@ -9,6 +9,7 @@
 
 module Stack.Build
   (build
+  ,buildLocalTargets
   ,loadPackage
   ,mkBaseConfigOpts
   ,queryBuildInfo
@@ -39,6 +40,7 @@ import           Stack.Build.Execute
 import           Stack.Build.Installed
 import           Stack.Build.Source
 import           Stack.Package
+import           Stack.Setup (withNewLocalBuildTargets)
 import           Stack.Types.Build
 import           Stack.Types.Config
 import           Stack.Types.NamedComponent
@@ -68,11 +70,22 @@ build msetLocalFiles = do
 
     checkSubLibraryDependencies (Map.elems $ smProject sourceMap)
 
+    boptsCli <- view $ envConfigL.to envConfigBuildOptsCLI
     -- Set local files, necessary for file watching
     stackYaml <- view stackYamlL
     for_ msetLocalFiles $ \setLocalFiles -> do
-      files <- sequence
-        [lpFiles lp | lp <- allLocals]
+      files <-
+        if boptsCLIWatchAll boptsCli
+        then sequence [lpFiles lp | lp <- allLocals]
+        else forM allLocals $ \lp -> do
+          let pn = packageName (lpPackage lp)
+          case Map.lookup pn (smtTargets $ smTargets sourceMap) of
+            Nothing ->
+              pure Set.empty
+            Just (TargetAll _) ->
+              lpFiles lp
+            Just (TargetComps components) ->
+              lpFilesForComponents components lp
       liftIO $ setLocalFiles $ Set.insert stackYaml $ Set.unions files
 
     checkComponentsBuildable allLocals
@@ -81,7 +94,6 @@ build msetLocalFiles = do
     (installedMap, globalDumpPkgs, snapshotDumpPkgs, localDumpPkgs) <-
         getInstalled installMap
 
-    boptsCli <- view $ envConfigL.to envConfigBuildOptsCLI
     baseConfigOpts <- mkBaseConfigOpts boptsCli
     plan <- constructPlan baseConfigOpts localDumpPkgs loadPackage sourceMap installedMap (boptsCLIInitialBuildSteps boptsCli)
 
@@ -107,6 +119,10 @@ build msetLocalFiles = do
                          (smtTargets $ smTargets sourceMap)
                          plan
 
+buildLocalTargets :: HasEnvConfig env => NonEmpty Text -> RIO env (Either SomeException ())
+buildLocalTargets targets =
+  tryAny $ withNewLocalBuildTargets (NE.toList targets) $ build Nothing
+
 justLocals :: Plan -> [PackageIdentifier]
 justLocals =
     map taskProvides .
