@@ -12,6 +12,7 @@ module Stack.Build.Cache
     , tryGetConfigCache
     , tryGetCabalMod
     , tryGetSetupConfigMod
+    , tryGetPackageProjectRoot
     , getInstalledExes
     , tryGetFlagCache
     , deleteCaches
@@ -21,6 +22,8 @@ module Stack.Build.Cache
     , writeBuildCache
     , writeConfigCache
     , writeCabalMod
+    , writeSetupConfigMod
+    , writePackageProjectRoot
     , TestStatus (..)
     , setTestStatus
     , getTestStatus
@@ -33,6 +36,7 @@ module Stack.Build.Cache
 import           Stack.Prelude
 import           Crypto.Hash (hashWith, SHA256(..))
 import qualified Data.ByteArray as Mem (convert)
+import           Data.ByteString.Builder (byteString)
 import qualified Data.Map as M
 import qualified Data.Set as Set
 import qualified Data.Text as T
@@ -153,6 +157,18 @@ tryGetFileMod fp =
   liftIO $ either (const Nothing) (Just . modificationTime) <$>
       tryIO (getFileStatus fp)
 
+-- | Try to read the project root from the last build of a package
+tryGetPackageProjectRoot :: HasEnvConfig env
+               => Path Abs Dir -> RIO env (Maybe ByteString)
+tryGetPackageProjectRoot dir = do
+  fp <- toFilePath <$> configPackageProjectRoot dir
+  tryReadFileBinary fp
+
+tryReadFileBinary :: MonadIO m => FilePath -> m (Maybe ByteString)
+tryReadFileBinary fp =
+  liftIO $ either (const Nothing) Just <$>
+      tryIO (readFileBinary fp)
+
 -- | Write the dirtiness cache for this package's files.
 writeBuildCache :: HasEnvConfig env
                 => Path Abs Dir
@@ -182,6 +198,30 @@ writeCabalMod dir x = do
     writeBinaryFileAtomic fp "Just used for its modification time"
     liftIO $ setFileTimes (toFilePath fp) x x
 
+-- | See 'tryGetSetupConfigMod'
+writeSetupConfigMod
+  :: HasEnvConfig env
+  => Path Abs Dir
+  -> Maybe CTime
+  -> RIO env ()
+writeSetupConfigMod dir Nothing = do
+    fp <- configSetupConfigMod dir
+    ignoringAbsence $ removeFile fp
+writeSetupConfigMod dir (Just x) = do
+    fp <- configSetupConfigMod dir
+    writeBinaryFileAtomic fp "Just used for its modification time"
+    liftIO $ setFileTimes (toFilePath fp) x x
+
+-- | See 'tryGetPackageProjectRoot'
+writePackageProjectRoot
+  :: HasEnvConfig env
+  => Path Abs Dir
+  -> ByteString
+  -> RIO env ()
+writePackageProjectRoot dir projectRoot = do
+    fp <- configPackageProjectRoot dir
+    writeBinaryFileAtomic fp (byteString projectRoot)
+
 -- | Delete the caches for the project.
 deleteCaches :: HasEnvConfig env => Path Abs Dir -> RIO env ()
 deleteCaches dir
