@@ -6,6 +6,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE LambdaCase #-}
 
 -- | The general Stack configuration that starts everything off. This should
 -- be smart to falback if there is no stack.yaml, instead relying on
@@ -30,11 +31,13 @@ module Stack.Config
   ,defaultConfigYaml
   ,getProjectConfig
   ,withBuildConfig
+  ,withNewLogFunc
   ) where
 
 import           Control.Monad.Extra (firstJustM)
 import           Stack.Prelude
 import           Pantry.Internal.AesonExtended
+import           Data.Array.IArray ((!), (//))
 import qualified Data.ByteString as S
 import           Data.ByteString.Builder (byteString)
 import           Data.Coerce (coerce)
@@ -49,9 +52,8 @@ import           Distribution.System (OS (..), Platform (..), buildPlatform, Arc
 import qualified Distribution.Text
 import           Distribution.Version (simplifyVersionRange, mkVersion')
 import           GHC.Conc (getNumProcessors)
-import           Lens.Micro ((.~))
 import           Network.HTTP.StackClient (httpJSON, parseUrlThrow, getResponseBody)
-import           Options.Applicative (Parser, strOption, long, help)
+import           Options.Applicative (Parser, help, long, metavar, strOption)
 import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
 import           Path.Find (findInParents)
@@ -74,14 +76,18 @@ import           Stack.Types.Nix
 import           Stack.Types.Resolver
 import           Stack.Types.SourceMap
 import           Stack.Types.Version
-import           System.Console.ANSI (hSupportsANSIWithoutEmulation)
+import           System.Console.ANSI (hSupportsANSIWithoutEmulation, setSGRCode)
 import           System.Environment
 import           System.Info.ShortPathName (getShortPathName)
 import           System.PosixCompat.Files (fileOwner, getFileStatus)
 import           System.PosixCompat.User (getEffectiveUserID)
 import           RIO.List (unzip)
-import           RIO.PrettyPrint (stylesUpdateL, useColorL)
+import           RIO.PrettyPrint (Style (Highlight, Secondary),
+                   logLevelToStyle, stylesUpdateL, useColorL)
+import           RIO.PrettyPrint.StylesUpdate (StylesUpdate (..))
+import           RIO.PrettyPrint.DefaultStyles (defaultStyles)
 import           RIO.Process
+import           RIO.Time (toGregorian)
 
 -- | If deprecated path exists, use it and print a warning.
 -- Otherwise, return the new path.
@@ -155,18 +161,18 @@ makeConcreteResolver ar = do
                 iopc <- loadConfigYaml (parseProjectAndConfigMonoid (parent fp)) fp
                 ProjectAndConfigMonoid project _ <- liftIO iopc
                 return $ projectResolver project
-            ARLatestNightly -> nightlySnapshotLocation . snapshotsNightly <$> getSnapshots
+            ARLatestNightly -> RSLSynonym . Nightly . snapshotsNightly <$> getSnapshots
             ARLatestLTSMajor x -> do
                 snapshots <- getSnapshots
                 case IntMap.lookup x $ snapshotsLts snapshots of
                     Nothing -> throwString $ "No LTS release found with major version " ++ show x
-                    Just y -> return $ ltsSnapshotLocation x y
+                    Just y -> return $ RSLSynonym $ LTS x y
             ARLatestLTS -> do
                 snapshots <- getSnapshots
                 if IntMap.null $ snapshotsLts snapshots
                    then throwString "No LTS releases found"
                    else let (x, y) = IntMap.findMax $ snapshotsLts snapshots
-                        in return $ ltsSnapshotLocation x y
+                        in return $ RSLSynonym $ LTS x y
     logInfo $ "Selected resolver: " <> display r
     return r
 
@@ -174,9 +180,9 @@ makeConcreteResolver ar = do
 getLatestResolver :: HasConfig env => RIO env RawSnapshotLocation
 getLatestResolver = do
     snapshots <- getSnapshots
-    let mlts = uncurry ltsSnapshotLocation <$>
+    let mlts = uncurry LTS <$>
                listToMaybe (reverse (IntMap.toList (snapshotsLts snapshots)))
-    pure $ fromMaybe (nightlySnapshotLocation (snapshotsNightly snapshots)) mlts
+    pure $ RSLSynonym $ fromMaybe (Nightly (snapshotsNightly snapshots)) mlts
 
 -- Interprets ConfigMonoid options.
 configFromConfigMonoid
@@ -224,6 +230,7 @@ configFromConfigMonoid
 
          configExtraIncludeDirs = configMonoidExtraIncludeDirs
          configExtraLibDirs = configMonoidExtraLibDirs
+         configCustomPreprocessorExts = configMonoidCustomPreprocessorExts
          configOverrideGccPath = getFirst configMonoidOverrideGccPath
 
          -- Only place in the codebase where platform is hard-coded. In theory
@@ -323,7 +330,6 @@ configFromConfigMonoid
          configSetupInfoInline = configMonoidSetupInfoInline
          configPvpBounds = fromFirst (PvpBounds PvpBoundsNone False) configMonoidPvpBounds
          configModifyCodePage = fromFirstTrue configMonoidModifyCodePage
-         configExplicitSetupDeps = configMonoidExplicitSetupDeps
          configRebuildGhcOptions = fromFirstFalse configMonoidRebuildGhcOptions
          configApplyGhcOptions = fromFirst AGOLocals configMonoidApplyGhcOptions
          configAllowNewer = fromFirst False configMonoidAllowNewer
@@ -353,10 +359,12 @@ configFromConfigMonoid
                 ColorNever  -> False
                 ColorAlways -> True
                 ColorAuto  -> useAnsi
-         configRunner = configRunner'
-             & processContextL .~ origEnv
-             & stylesUpdateL .~ stylesUpdate'
-             & useColorL .~ fromMaybe useColor' mUseColor
+         useColor'' = fromMaybe useColor' mUseColor
+         configRunner'' = configRunner'
+               & processContextL .~ origEnv
+               & stylesUpdateL .~ stylesUpdate'
+               & useColorL .~ useColor''
+         go = runnerGlobalOpts configRunner'
 
      hsc <-
        case getFirst configMonoidPackageIndices of
@@ -372,18 +380,72 @@ configFromConfigMonoid
              Just x -> pure x
          Nothing -> pure $ configStackRoot </> relDirPantry
 
+     let snapLoc =
+            case getFirst configMonoidSnapshotLocation of
+                Nothing -> defaultSnapshotLocation
+                Just addr -> customSnapshotLocation
+                                where
+                    customSnapshotLocation (LTS x y) =
+                        mkRSLUrl $ addr'
+                            <> "/lts/" <> display x
+                            <> "/" <> display y <> ".yaml"
+                    customSnapshotLocation (Nightly date) =
+                        let (year, month, day) = toGregorian date
+                        in mkRSLUrl $ addr'
+                            <> "/nightly/"
+                            <> display year
+                            <> "/" <> display month
+                            <> "/" <> display day <> ".yaml"
+                    mkRSLUrl builder = RSLUrl (utf8BuilderToText builder) Nothing
+                    addr' = display $ T.dropWhileEnd (=='/') addr
+
      let configStackDeveloperMode = fromFirst stackDeveloperModeDefault configMonoidStackDeveloperMode
 
-     withPantryConfig
-       pantryRoot
-       hsc
-       (maybe HpackBundled HpackCommand $ getFirst configMonoidOverrideHpack)
-       clConnectionCount
-       (fromFirst defaultCasaRepoPrefix configMonoidCasaRepoPrefix)
-       defaultCasaMaxPerRequest
-       (\configPantryConfig -> initUserStorage
-         (configStackRoot </> relFileStorage)
-         (\configUserStorage -> inner Config {..}))
+     withNewLogFunc go useColor'' stylesUpdate' $ \logFunc -> do
+       let configRunner = configRunner'' & logFuncL .~ logFunc
+       withLocalLogFunc logFunc $ withPantryConfig
+         pantryRoot
+         hsc
+         (maybe HpackBundled HpackCommand $ getFirst configMonoidOverrideHpack)
+         clConnectionCount
+         (fromFirst defaultCasaRepoPrefix configMonoidCasaRepoPrefix)
+         defaultCasaMaxPerRequest
+         snapLoc
+         (\configPantryConfig -> initUserStorage
+           (configStackRoot </> relFileStorage)
+           (\configUserStorage -> inner Config {..}))
+
+-- | Runs the provided action with the given 'LogFunc' in the environment
+withLocalLogFunc :: HasLogFunc env => LogFunc -> RIO env a -> RIO env a
+withLocalLogFunc logFunc = local (set logFuncL logFunc)
+
+-- | Runs the provided action with a new 'LogFunc', given a 'StylesUpdate'.
+withNewLogFunc :: MonadUnliftIO m
+               => GlobalOpts
+               -> Bool  -- ^ Use color
+               -> StylesUpdate
+               -> (LogFunc -> m a)
+               -> m a
+withNewLogFunc go useColor (StylesUpdate update) inner = do
+  logOptions0 <- logOptionsHandle stderr False
+  let logOptions
+        = setLogUseColor useColor
+        $ setLogLevelColors logLevelColors
+        $ setLogSecondaryColor secondaryColor
+        $ setLogAccentColors (const highlightColor)
+        $ setLogUseTime (globalTimeInLog go)
+        $ setLogMinLevel (globalLogLevel go)
+        $ setLogVerboseFormat (globalLogLevel go <= LevelDebug)
+        $ setLogTerminal (globalTerminal go)
+          logOptions0
+  withLogFunc logOptions inner
+ where
+  styles = defaultStyles // update
+  logLevelColors :: LogLevel -> Utf8Builder
+  logLevelColors level =
+    fromString $ setSGRCode $ snd $ styles ! logLevelToStyle level
+  secondaryColor = fromString $ setSGRCode $ snd $ styles ! Secondary
+  highlightColor = fromString $ setSGRCode $ snd $ styles ! Highlight
 
 -- | Get the default location of the local programs directory.
 getDefaultLocalProgramsBase :: MonadThrow m
@@ -591,6 +653,14 @@ fillProjectWanted stackYamlFP config project locCache snapCompiler snapPackages
       pp <- mkProjectPackage YesPrintWarnings resolved (boptsHaddock bopts)
       pure (cpName $ ppCommon pp, pp)
 
+    -- prefetch git repos to avoid cloning per subdirectory
+    -- see https://github.com/commercialhaskell/stack/issues/5411
+    let gitRepos = mapMaybe (\case
+                              (RPLImmutable (RPLIRepo repo rpm)) -> Just (repo, rpm)
+                              _ -> Nothing) (projectDependencies project)
+    logDebug ("Prefetching git repos: " <> display (T.pack (show gitRepos)))
+    fetchReposRaw gitRepos
+
     (deps0, mcompleted) <- fmap unzip . forM (projectDependencies project) $ \rpl -> do
       (pl, mCompleted) <- case rpl of
          RPLImmutable rpli -> do
@@ -886,7 +956,10 @@ getDefaultUserConfigPath stackRoot = do
     return path
 
 packagesParser :: Parser [String]
-packagesParser = many (strOption (long "package" <> help "Additional packages that must be installed"))
+packagesParser = many (strOption
+                   (long "package" <>
+                     metavar "PACKAGE(S)" <>
+                     help "Additional package(s) that must be installed"))
 
 defaultConfigYaml :: IsString s => s
 defaultConfigYaml =
