@@ -379,7 +379,7 @@ generateHpcMarkupIndex = do
                 encodeUtf8Builder (pathToHtml outputDir) <>
                 "\".</b>"
             else
-                "<table class=\"dashboard\" width=\"100%\" boder=\"1\"><tbody>" <>
+                "<table class=\"dashboard\" width=\"100%\" border=\"1\"><tbody>" <>
                 "<p><b>NOTE: This is merely a listing of the html files found in the coverage reports directory.  Some of these reports may be old.</b></p>" <>
                 "<tr><th>Package</th><th>TestSuite</th><th>Modification Time</th></tr>" <>
                 foldMap encodeUtf8Builder rows <>
@@ -434,7 +434,7 @@ findPackageFieldForBuiltPackage pkgDir pkgId internalLibs field = do
         extractField path = do
             contents <- readFileUtf8 (toFilePath path)
             case asum (map (T.stripPrefix (field <> ": ")) (T.lines contents)) of
-                Just result -> return $ Right result
+                Just result -> return $ Right $ T.strip result
                 Nothing -> notFoundErr
     cabalVer <- view cabalVersionL
     if cabalVer < mkVersion [1, 24]
