@@ -21,6 +21,7 @@ import           Data.ByteString.Builder (byteString)
 import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy as LBS
 import           Data.List
+import qualified Data.List.NonEmpty as NE
 import qualified Data.Map.Strict as M
 import qualified Data.Set as S
 import qualified Data.Text as T
@@ -32,7 +33,7 @@ import           Path.Extra (toFilePathNoTrailingSep)
 import           Path.IO hiding (withSystemTempDir)
 import qualified RIO
 import           RIO.PrettyPrint
-import           RIO.Process (HasProcessContext, exec, proc, readProcess_)
+import           RIO.Process (HasProcessContext, exec, proc, readProcess_, withWorkingDir)
 import           Stack.Build
 import           Stack.Build.Installed
 import           Stack.Build.Source
@@ -41,7 +42,6 @@ import           Stack.Constants
 import           Stack.Constants.Config
 import           Stack.Ghci.Script
 import           Stack.Package
-import           Stack.Setup (withNewLocalBuildTargets)
 import           Stack.Types.Build
 import           Stack.Types.Config
 import           Stack.Types.NamedComponent
@@ -346,14 +346,17 @@ buildDepsAndInitialSteps GhciOpts{..} localTargets = do
     let targets = localTargets ++ map T.pack ghciAdditionalPackages
     -- If necessary, do the build, for local packagee targets, only do
     -- 'initialBuildSteps'.
-    when (not ghciNoBuild && not (null targets)) $ do
-        -- only new local targets could appear here
-        eres <- tryAny $ withNewLocalBuildTargets targets $ build Nothing
+    case NE.nonEmpty targets of
+      -- only new local targets could appear here
+      Just nonEmptyTargets | not ghciNoBuild -> do
+        eres <- buildLocalTargets nonEmptyTargets
         case eres of
             Right () -> return ()
             Left err -> do
                 prettyError $ fromString (show err)
                 prettyWarn "Build failed, but trying to launch GHCi anyway"
+      _ ->
+        return ()
 
 checkAdditionalPackages :: MonadThrow m => [String] -> m [PackageName]
 checkAdditionalPackages pkgs = forM pkgs $ \name -> do
@@ -412,7 +415,7 @@ runGhci GhciOpts{..} targets mainFile pkgs extraFiles exposePackages = do
     compilerExeName <- view $ compilerPathsL.to cpCompiler.to toFilePath
     let execGhci extras = do
             menv <- liftIO $ configProcessContextSettings config defaultEnvSettings
-            withProcessContext menv $ exec
+            withPackageWorkingDir $ withProcessContext menv $ exec
                  (fromMaybe compilerExeName ghciGhcCommand)
                  (("--interactive" : ) $
                  -- This initial "-i" resets the include directories to
@@ -420,6 +423,10 @@ runGhci GhciOpts{..} targets mainFile pkgs extraFiles exposePackages = do
                  -- is included.
                   (if null pkgs then id else ("-i" : )) $
                   odir <> pkgopts <> extras <> ghciGhcOptions <> ghciArgs)
+        withPackageWorkingDir =
+            case pkgs of
+              [pkg] -> withWorkingDir (toFilePath $ ghciPkgDir pkg)
+              _ -> id
         -- TODO: Consider optimizing this check. Perhaps if no
         -- "with-ghc" is specified, assume that it is not using intero.
         checkIsIntero =
@@ -637,11 +644,12 @@ loadGhciPkgDesc buildOptsCLI name cabalfp target = do
           (cpCabalConfigOpts . ppCommon <$> M.lookup name smProject)
           <|>
           (cpCabalConfigOpts . dpCommon <$> M.lookup name smDeps)
+        sourceMapFlags = maybe mempty (cpFlags . ppCommon) $ M.lookup name smProject
         config =
             PackageConfig
             { packageConfigEnableTests = True
             , packageConfigEnableBenchmarks = True
-            , packageConfigFlags = getLocalFlags buildOptsCLI name
+            , packageConfigFlags = getLocalFlags buildOptsCLI name `M.union` sourceMapFlags
             , packageConfigGhcOptions = sourceMapGhcOptions
             , packageConfigCabalConfigOpts = sourceMapCabalConfigOpts
             , packageConfigCompilerVersion = compilerVersion
@@ -747,48 +755,63 @@ wantedPackageComponents _ _ _ = S.empty
 
 checkForIssues :: HasLogFunc env => [GhciPkgInfo] -> RIO env ()
 checkForIssues pkgs = do
-    unless (null issues) $ borderedWarning $ do
-        logWarn "Warning: There are cabal settings for this project which may prevent GHCi from loading your code properly."
-        logWarn "In some cases it can also load some projects which would otherwise fail to build."
-        logWarn ""
-        mapM_ (logWarn . RIO.display) $ intercalate [""] issues
-        logWarn ""
-        logWarn "To resolve, remove the flag(s) from the cabal file(s) and instead put them at the top of the haskell files."
-        logWarn ""
+    when (length pkgs > 1) $ borderedWarning $ do
+        -- Cabal flag issues could arise only when there are at least 2 packages
+        unless (null cabalFlagIssues) $ borderedWarning $ do
+            logWarn "Warning: There are cabal flags for this project which may prevent GHCi from loading your code properly."
+            logWarn "In some cases it can also load some projects which would otherwise fail to build."
+            logWarn ""
+            mapM_ (logWarn . RIO.display) $ intercalate [""] cabalFlagIssues
+            logWarn ""
+            logWarn "To resolve, remove the flag(s) from the cabal file(s) and instead put them at the top of the haskell files."
+            logWarn ""
         logWarn "It isn't yet possible to load multiple packages into GHCi in all cases - see"
         logWarn "https://ghc.haskell.org/trac/ghc/ticket/10827"
   where
-    issues = concat
-        [ mixedFlag "-XNoImplicitPrelude"
-          [ "-XNoImplicitPrelude will be used, but GHCi will likely fail to build things which depend on the implicit prelude." ]
-        , mixedFlag "-XCPP"
-          [ "-XCPP will be used, but it can cause issues with multiline strings."
-          , "See https://downloads.haskell.org/~ghc/7.10.2/docs/html/users_guide/options-phases.html#cpp-string-gaps"
-          ]
-        , mixedFlag "-XNoTraditionalRecordSyntax"
-          [ "-XNoTraditionalRecordSyntax will be used, but it break modules which use record syntax." ]
-        , mixedFlag "-XTemplateHaskell"
-          [ "-XTemplateHaskell will be used, but it may cause compilation issues due to different parsing of '$' when there's no space after it." ]
-        , mixedFlag "-XQuasiQuotes"
-          [ "-XQuasiQuotes will be used, but it may cause parse failures due to a different meaning for list comprehension syntax like [x| ... ]" ]
-        , mixedFlag "-XSafe"
-          [ "-XSafe will be used, but it will fail to compile unsafe modules." ]
-        , mixedFlag "-XArrows"
-          [ "-XArrows will be used, but it will cause non-arrow usages of proc, (-<), (-<<) to fail" ]
-        , mixedFlag "-XOverloadedStrings"
-          [ "-XOverloadedStrings will be used, but it can cause type ambiguity in code not usually compiled with it." ]
-        , mixedFlag "-XOverloadedLists"
-          [ "-XOverloadedLists will be used, but it can cause type ambiguity in code not usually compiled with it." ]
-        , mixedFlag "-XMonoLocalBinds"
-          [ "-XMonoLocalBinds will be used, but it can cause type errors in code which expects generalized local bindings." ]
-        , mixedFlag "-XTypeFamilies"
-          [ "-XTypeFamilies will be used, but it implies -XMonoLocalBinds, and so can cause type errors in code which expects generalized local bindings." ]
-        , mixedFlag "-XGADTs"
-          [ "-XGADTs will be used, but it implies -XMonoLocalBinds, and so can cause type errors in code which expects generalized local bindings." ]
-        , mixedFlag "-XNewQualifiedOperators"
-          [ "-XNewQualifiedOperators will be used, but this will break usages of the old qualified operator syntax." ]
+    cabalFlagIssues = concatMap mixedFlag
+        [ ( "-XNoImplicitPrelude"
+          , [ "-XNoImplicitPrelude will be used, but GHCi will likely fail to build things which depend on the implicit prelude."]
+          )
+        , ( "-XCPP"
+          , [ "-XCPP will be used, but it can cause issues with multiline strings."
+            , "See https://downloads.haskell.org/~ghc/7.10.2/docs/html/users_guide/options-phases.html#cpp-string-gaps"
+            ]
+          )
+        , ( "-XNoTraditionalRecordSyntax"
+          , [ "-XNoTraditionalRecordSyntax will be used, but it break modules which use record syntax." ]
+          )
+        , ( "-XTemplateHaskell"
+          , [ "-XTemplateHaskell will be used, but it may cause compilation issues due to different parsing of '$' when there's no space after it." ]
+          )
+        , ( "-XQuasiQuotes"
+          , [ "-XQuasiQuotes will be used, but it may cause parse failures due to a different meaning for list comprehension syntax like [x| ... ]" ]
+          )
+        , ( "-XSafe"
+          , [ "-XSafe will be used, but it will fail to compile unsafe modules." ]
+          )
+        , ( "-XArrows"
+          , [ "-XArrows will be used, but it will cause non-arrow usages of proc, (-<), (-<<) to fail" ]
+          )
+        , ( "-XOverloadedStrings"
+          , [ "-XOverloadedStrings will be used, but it can cause type ambiguity in code not usually compiled with it." ]
+          )
+        , ( "-XOverloadedLists"
+          , [ "-XOverloadedLists will be used, but it can cause type ambiguity in code not usually compiled with it." ]
+          )
+        , ( "-XMonoLocalBinds"
+          , [ "-XMonoLocalBinds will be used, but it can cause type errors in code which expects generalized local bindings." ]
+          )
+        , ( "-XTypeFamilies"
+          , [ "-XTypeFamilies will be used, but it implies -XMonoLocalBinds, and so can cause type errors in code which expects generalized local bindings." ]
+          )
+        , ( "-XGADTs"
+          , [ "-XGADTs will be used, but it implies -XMonoLocalBinds, and so can cause type errors in code which expects generalized local bindings." ]
+          )
+        , ( "-XNewQualifiedOperators"
+          , [ "-XNewQualifiedOperators will be used, but this will break usages of the old qualified operator syntax." ]
+          )
         ]
-    mixedFlag flag msgs =
+    mixedFlag (flag, msgs) =
         let x = partitionComps (== flag) in
         [ msgs ++ showWhich x | mixedSettings x ]
     mixedSettings (xs, ys) = xs /= [] && ys /= []
@@ -823,9 +846,11 @@ checkForDuplicateModules :: HasTerm env => [GhciPkgInfo] -> RIO env ()
 checkForDuplicateModules pkgs = do
     unless (null duplicates) $ do
         borderedWarning $ do
-            prettyError $ "Multiple files use the same module name:" <>
+            prettyWarn $ "Multiple files use the same module name:" <>
               line <> bulletedList (map prettyDuplicate duplicates)
-        throwM LoadingDuplicateModules
+        -- MSS 2020-10-13 Disabling, may remove entirely in the future
+        -- See: https://github.com/commercialhaskell/stack/issues/5407#issuecomment-707339928
+        -- throwM LoadingDuplicateModules
   where
     duplicates :: [(ModuleName, Map (Path Abs File) (Set (PackageName, NamedComponent)))]
     duplicates =
