@@ -20,9 +20,9 @@ import qualified System.FilePath as FilePath
 configOptsParser :: FilePath -> GlobalOptsContext -> Parser ConfigMonoid
 configOptsParser currentDir hide0 =
     (\stackRoot workDir buildOpts dockerOpts nixOpts systemGHC installGHC arch
-        ghcVariant ghcBuild jobs includes libs overrideGccPath overrideHpack
+        ghcVariant ghcBuild jobs includes libs preprocs overrideGccPath overrideHpack
         skipGHCCheck skipMsys localBin setupInfoLocations modifyCodePage
-        allowDifferentUser dumpLogs colorWhen -> mempty
+        allowDifferentUser dumpLogs colorWhen snapLoc -> mempty
             { configMonoidStackRoot = stackRoot
             , configMonoidWorkDir = workDir
             , configMonoidBuildOpts = buildOpts
@@ -37,6 +37,7 @@ configOptsParser currentDir hide0 =
             , configMonoidJobs = jobs
             , configMonoidExtraIncludeDirs = includes
             , configMonoidExtraLibDirs = libs
+            , configMonoidCustomPreprocessorExts = preprocs
             , configMonoidOverrideGccPath = overrideGccPath
             , configMonoidOverrideHpack = overrideHpack
             , configMonoidSkipMsys = skipMsys
@@ -46,6 +47,7 @@ configOptsParser currentDir hide0 =
             , configMonoidAllowDifferentUser = allowDifferentUser
             , configMonoidDumpLogs = dumpLogs
             , configMonoidColorWhen = colorWhen
+            , configMonoidSnapshotLocation = snapLoc
             })
     <$> optionalFirst (absDirOption
             ( long stackRootOptionName
@@ -102,6 +104,12 @@ configOptsParser currentDir hide0 =
            <> help "Extra directories to check for libraries"
            <> hide
             ))
+    <*> many (strOption
+            ( long "custom-preprocessor-extensions"
+           <> metavar "EXT"
+           <> help "Extensions used for custom preprocessors"
+           <> hide
+            ))
     <*> optionalFirst (absFileOption
              ( long "with-gcc"
             <> metavar "PATH-TO-GCC"
@@ -161,6 +169,11 @@ configOptsParser currentDir hide0 =
                     \support color codes"
             <> hide
              ))
+    <*> optionalFirst (strOption
+            ( long "snapshot-location-base"
+           <> help "The base location of LTS/Nightly snapshots"
+           <> metavar "URL"
+            ))
   where
     hide = hideMods (hide0 /= OuterGlobalOpts)
     toDumpLogs (First (Just True)) = First (Just DumpAllLogs)
