@@ -13,27 +13,32 @@ import           Stack.Types.Config
 testOptsParser :: Bool -> Parser TestOptsMonoid
 testOptsParser hide0 =
     TestOptsMonoid
-        <$> firstBoolFlags
+        <$> firstBoolFlagsTrue
                 "rerun-tests"
                 "running already successful tests"
                 hide
         <*> fmap
-                (fromMaybe [])
-                (optional
+                concat
+                (many
                     (argsOption
                         (long "test-arguments" <>
                          long "ta" <>
                          metavar "TEST_ARGS" <>
                          help "Arguments passed in to the test suite program" <>
                          hide)))
-        <*> optionalFirst
+        <*> optionalFirstFalse
                 (flag' True
                     (long "coverage" <>
                      help "Generate a code coverage report" <>
                      hide))
-        <*> optionalFirst
+        <*> optionalFirstFalse
                 (flag' True
                     (long "no-run-tests" <>
                      help "Disable running of tests. (Tests will still be built.)" <>
                      hide))
+        <*> optionalFirst
+                (option (fmap Just auto)
+                    (long "test-suite-timeout" <>
+                     help "Maximum test suite run time in seconds." <>
+                     hide))
    where hide = hideMods hide0
