@@ -82,6 +82,7 @@ import qualified RIO.PrettyPrint as PP (Style (Module))
 data Ctx = Ctx { ctxFile :: !(Path Abs File)
                , ctxDistDir :: !(Path Abs Dir)
                , ctxBuildConfig :: !BuildConfig
+               , ctxCabalVer :: !Version
                }
 
 instance HasPlatform Ctx
@@ -219,9 +220,10 @@ packageFromPackageDescription packageConfig pkgFlags (PackageDescriptionPair pkg
              let pkgDir = parent cabalfp
              distDir <- distDirFromDir pkgDir
              bc <- view buildConfigL
+             cabalVer <- view cabalVersionL
              (componentModules,componentFiles,dataFiles',warnings) <-
                  runRIO
-                     (Ctx cabalfp distDir bc)
+                     (Ctx cabalfp distDir bc cabalVer)
                      (packageDescModulesAndFiles pkg)
              setupFiles <-
                  if buildType pkg == Custom
@@ -795,13 +797,21 @@ resolveComponentFiles
 resolveComponentFiles component build names = do
     dirs <- mapMaybeM resolveDirOrWarn (hsSourceDirs build)
     dir <- asks (parent . ctxFile)
+    agdirs <- autogenDirs
     (modules,files,warnings) <-
         resolveFilesAndDeps
             component
-            (if null dirs then [dir] else dirs)
+            ((if null dirs then [dir] else dirs) ++ agdirs)
             names
     cfiles <- buildOtherSources build
     return (modules, files <> cfiles, warnings)
+  where
+    autogenDirs = do
+      cabalVer <- asks ctxCabalVer
+      distDir <- asks ctxDistDir
+      let compDir = componentAutogenDir cabalVer component distDir
+          pkgDir = maybeToList $ packageAutogenDir cabalVer distDir
+      filterM doesDirExist $ compDir : pkgDir
 
 -- | Get all C sources and extra source files in a build.
 buildOtherSources :: BuildInfo -> RIO Ctx [DotCabalPath]
@@ -1097,7 +1107,7 @@ parseHI
     :: FilePath -> RIO Ctx (Set ModuleName, [Path Abs File])
 parseHI hiPath = do
   dir <- asks (parent . ctxFile)
-  result <- liftIO $ Iface.fromFile hiPath
+  result <- liftIO $ Iface.fromFile hiPath `catchAnyDeep` \e -> pure (Left (show e))
   case result of
     Left msg -> do
       prettyStackDevL
@@ -1163,7 +1173,9 @@ findCandidate
     -> RIO Ctx (Maybe DotCabalPath)
 findCandidate dirs name = do
     pkg <- asks ctxFile >>= parsePackageNameFromFilePath
-    candidates <- liftIO makeNameCandidates
+    customPreprocessorExts <- view $ configL . to configCustomPreprocessorExts
+    let haskellPreprocessorExts = haskellDefaultPreprocessorExts ++ customPreprocessorExts
+    candidates <- liftIO $ makeNameCandidates haskellPreprocessorExts
     case candidates of
         [candidate] -> return (Just (cons candidate))
         [] -> do
@@ -1183,11 +1195,12 @@ findCandidate dirs name = do
             DotCabalFile{} -> DotCabalFilePath
             DotCabalCFile{} -> DotCabalCFilePath
     paths_pkg pkg = "Paths_" ++ packageNameString pkg
-    makeNameCandidates =
-        liftM (nubOrd . concat) (mapM makeDirCandidates dirs)
-    makeDirCandidates :: Path Abs Dir
+    makeNameCandidates haskellPreprocessorExts =
+        liftM (nubOrd . concat) (mapM (makeDirCandidates haskellPreprocessorExts) dirs)
+    makeDirCandidates :: [Text]
+                      -> Path Abs Dir
                       -> IO [Path Abs File]
-    makeDirCandidates dir =
+    makeDirCandidates haskellPreprocessorExts dir =
         case name of
             DotCabalMain fp -> resolveCandidate dir fp
             DotCabalFile fp -> resolveCandidate dir fp
@@ -1258,8 +1271,8 @@ logPossibilities dirs mn = do
         , flow "but did find:"
         , line <> bulletedList (map pretty possibilities)
         , flow "If you are using a custom preprocessor for this module"
-        , flow "with its own file extension, consider adding the file(s)"
-        , flow "to your .cabal under extra-source-files."
+        , flow "with its own file extension, consider adding the extension"
+        , flow "to the 'custom-preprocessor-extensions' field in stack.yaml."
         ]
   where
     makePossibilities name =
