@@ -167,7 +167,7 @@ paths =
     , ( "Directory containing binaries specific to a particular compiler (e.g. intero)"
       , "compiler-tools-bin"
       , WithoutHaddocks $ T.pack . toFilePathNoTrailingSep . piToolsDir )
-    , ( "Local bin dir where stack installs executables (e.g. ~/.local/bin)"
+    , ( "Local bin dir where stack installs executables (e.g. ~/.local/bin (Unix-like OSs) or %APPDATA%\\local\\bin (Windows))"
       , "local-bin"
       , WithoutHaddocks $ view $ configL.to configLocalBin.to toFilePathNoTrailingSep.to T.pack)
     , ( "Extra include directories"
