@@ -111,14 +111,10 @@ reexec inner = do
     -- Want to use Nix
     (True, False) -> do
       whenM getInContainer $ throwString "Cannot use Nix from within a Docker container"
-      inShell <- getInNixShell
-      if inShell
-        then do
-          isReexec <- view reExecL
-          if isReexec
-            then inner
-            else throwString "In Nix shell but reExecL is False"
-        else Nix.runShellAndExit
+      isReexec <- view reExecL
+      if isReexec
+      then inner
+      else Nix.runShellAndExit
 
     -- Want to use Docker
     (False, True) -> do
@@ -148,15 +144,8 @@ withRunnerGlobal go inner = do
                                     <$> getTerminalWidth)
                                    pure (globalTermWidth go)
   menv <- mkDefaultProcessContext
-  logOptions0 <- logOptionsHandle stderr False
-  let logOptions
-        = setLogUseColor useColor
-        $ setLogUseTime (globalTimeInLog go)
-        $ setLogMinLevel (globalLogLevel go)
-        $ setLogVerboseFormat (globalLogLevel go <= LevelDebug)
-        $ setLogTerminal (globalTerminal go)
-          logOptions0
-  withLogFunc logOptions $ \logFunc -> runRIO Runner
+  let update = globalStylesUpdate go
+  withNewLogFunc go useColor update $ \logFunc -> runRIO Runner
     { runnerGlobalOpts = go
     , runnerUseColor = useColor
     , runnerLogFunc = logFunc
