@@ -54,10 +54,13 @@ import qualified    Distribution.System as Cabal
 import              Distribution.Text (simpleParse)
 import              Distribution.Types.PackageName (mkPackageName)
 import              Distribution.Version (mkVersion)
+import              Network.HTTP.Client (redirectCount)
 import              Network.HTTP.StackClient (CheckHexDigest (..), HashCheck (..),
                                               getResponseBody, getResponseStatusCode, httpLbs, httpJSON,
                                               mkDownloadRequest, parseRequest, parseUrlThrow, setGithubHeaders,
-                                              setHashChecks, setLengthCheck, verifiedDownloadWithProgress, withResponse)
+                                              setHashChecks, setLengthCheck, verifiedDownloadWithProgress, withResponse,
+                                              setRequestMethod)
+import              Network.HTTP.Simple (getResponseHeader)
 import              Path hiding (fileExtension)
 import              Path.CheckInstall (warnInstallSearchPathIssues)
 import              Path.Extended (fileExtension)
@@ -465,9 +468,9 @@ warnUnsupportedCompiler ghcVersion = do
         logWarn "For more information, see: https://github.com/commercialhaskell/stack/issues/648"
         logWarn ""
         pure True
-    | ghcVersion >= mkVersion [8, 11] -> do
+    | ghcVersion >= mkVersion [9, 1] -> do
         logWarn $
-          "Stack has not been tested with GHC versions above 8.10, and using " <>
+          "Stack has not been tested with GHC versions above 9.0, and using " <>
           fromString (versionString ghcVersion) <>
           ", this may fail"
         pure True
@@ -492,9 +495,9 @@ warnUnsupportedCompilerCabal cp didWarn = do
         logWarn "This invocation will most likely fail."
         logWarn "To fix this, either use an older version of Stack or a newer resolver"
         logWarn "Acceptable resolvers: lts-3.0/nightly-2015-05-05 or later"
-    | cabalVersion >= mkVersion [3, 3] ->
+    | cabalVersion >= mkVersion [3, 5] ->
         logWarn $
-          "Stack has not been tested with Cabal versions above 3.2, but version " <>
+          "Stack has not been tested with Cabal versions above 3.4, but version " <>
           fromString (versionString cabalVersion) <>
           " was found, this may fail"
     | otherwise -> pure ()
@@ -525,7 +528,7 @@ ensureMsys sopts getSetupInfo' = do
                               Just x -> return x
                               Nothing -> throwString $ "MSYS2 not found for " ++ T.unpack osKey
                       let tool = Tool (PackageIdentifier (mkPackageName "msys2") version)
-                      Just <$> downloadAndInstallTool (configLocalPrograms config) info tool (installMsys2Windows osKey si)
+                      Just <$> downloadAndInstallTool (configLocalPrograms config) info tool (installMsys2Windows si)
                   | otherwise -> do
                       logWarn "Continuing despite missing tool: msys2"
                       return Nothing
@@ -664,18 +667,36 @@ ensureSandboxedCompiler sopts getSetupInfo' = do
         WCGhc version -> pure ["ghc-" ++ versionString version, "ghc"]
         WCGhcGit{} -> pure ["ghc"]
         WCGhcjs{} -> throwIO GhcjsNotSupported
+
+    -- Previously, we used findExecutable to locate these executables. This was
+    -- actually somewhat sloppy, as it could discover executables outside of the
+    -- sandbox. This led to a specific issue on Windows with GHC 9.0.1. See
+    -- https://gitlab.haskell.org/ghc/ghc/-/issues/20074. Instead, now, we look
+    -- on the paths specified only.
     let loop [] = do
           logError $ "Looked for sandboxed compiler named one of: " <> displayShow names
           logError $ "Could not find it on the paths " <> displayShow (edBins paths)
           throwString "Could not find sandboxed compiler"
         loop (x:xs) = do
-          res <- findExecutable x
+          res <- liftIO $ D.findExecutablesInDirectories (map toFilePath (edBins paths)) x
           case res of
-            Left _ -> loop xs
-            Right y -> parseAbsFile y
-    compiler <- withProcessContext menv $ loop names
+            [] -> loop xs
+            compiler:rest -> do
+              unless (null rest) $ do
+                logWarn "Found multiple candidate compilers:"
+                for_ res $ \y -> logWarn $ "- " <> fromString y
+                logWarn $ "This usually indicates a failed installation. Trying anyway with " <> fromString compiler
+              parseAbsFile compiler
+    compiler <- withProcessContext menv $ do
+      compiler <- loop names
+
+      -- Run this here to ensure that the sanity check uses the modified
+      -- environment, otherwise we may infect GHC_PACKAGE_PATH and break sanity
+      -- checks.
+      when (soptsSanityCheck sopts) $ sanityCheck compiler
+
+      pure compiler
 
-    when (soptsSanityCheck sopts) $ sanityCheck compiler
     cp <- pathsFromCompiler wc compilerBuild True compiler
     pure (cp, paths)
 
@@ -815,15 +836,8 @@ buildGhcFromSource getSetupInfo' installed (CompilerRepository url) commitId fla
    if compilerTool `elem` installed
      then return (compilerTool,CompilerBuildStandard)
      else do
-       let repo = Repo
-            { repoCommit = commitId
-            , repoUrl    = url
-            , repoType   = RepoGit
-            , repoSubdir = mempty
-            }
-
        -- clone the repository and execute the given commands
-       Pantry.withRepo repo $ do
+       Pantry.withRepo (Pantry.SimpleRepo url commitId RepoGit) $ do
          -- withRepo is guaranteed to set workingDirL, so let's get it
          mcwd <- traverse parseAbsDir =<< view workingDirL
          let cwd = fromMaybe (error "Invalid working directory") mcwd
@@ -836,9 +850,12 @@ buildGhcFromSource getSetupInfo' installed (CompilerRepository url) commitId fla
                , "--flavour=" <> flavour -- selected flavour
                , "binary-dist"
                ]
-           hadrianCmd
-             | osIsWindows = hadrianCmdWindows
-             | otherwise   = hadrianCmdPosix
+           hadrianScripts
+             | osIsWindows = hadrianScriptsWindows
+             | otherwise   = hadrianScriptsPosix
+
+         foundHadrianPaths <- filterM doesFileExist $ (cwd </>) <$> hadrianScripts
+         hadrianPath <- maybe (throwString "No Hadrian build script found") pure $ listToMaybe foundHadrianPaths
 
          logSticky $ "Building GHC from source with `"
             <> RIO.display flavour
@@ -847,7 +864,7 @@ buildGhcFromSource getSetupInfo' installed (CompilerRepository url) commitId fla
          -- We need to provide an absolute path to the script since
          -- the process package only sets working directory _after_
          -- discovering the executable
-         proc (toFilePath (cwd </> hadrianCmd)) hadrianArgs runProcess_
+         proc (toFilePath hadrianPath) hadrianArgs runProcess_
 
          -- find the bindist and install it
          bindistPath <- parseRelDir "_build/bindist"
@@ -872,8 +889,8 @@ buildGhcFromSource getSetupInfo' installed (CompilerRepository url) commitId fla
                              }
                    ghcdlinfo = GHCDownloadInfo mempty mempty dlinfo
                    installer
-                      | osIsWindows = installGHCWindows Nothing
-                      | otherwise   = installGHCPosix Nothing ghcdlinfo
+                      | osIsWindows = installGHCWindows
+                      | otherwise   = installGHCPosix ghcdlinfo
                si <- runMemoized getSetupInfo'
                _ <- downloadAndInstallTool
                  (configLocalPrograms config)
@@ -1115,23 +1132,19 @@ downloadAndInstallCompiler :: (HasBuildConfig env, HasGHCVariant env)
                            -> VersionCheck
                            -> Maybe String
                            -> RIO env Tool
-downloadAndInstallCompiler ghcBuild si wanted@WCGhc{} versionCheck mbindistURL = do
+downloadAndInstallCompiler ghcBuild si wanted@(WCGhc version) versionCheck mbindistURL = do
     ghcVariant <- view ghcVariantL
     (selectedVersion, downloadInfo) <- case mbindistURL of
         Just bindistURL -> do
             case ghcVariant of
                 GHCCustom _ -> return ()
                 _ -> throwM RequireCustomGHCVariant
-            case wanted of
-                WCGhc version ->
-                    return (version, GHCDownloadInfo mempty mempty DownloadInfo
-                             { downloadInfoUrl = T.pack bindistURL
-                             , downloadInfoContentLength = Nothing
-                             , downloadInfoSha1 = Nothing
-                             , downloadInfoSha256 = Nothing
-                             })
-                _ ->
-                    throwM WantedMustBeGHC
+            return (version, GHCDownloadInfo mempty mempty DownloadInfo
+                     { downloadInfoUrl = T.pack bindistURL
+                     , downloadInfoContentLength = Nothing
+                     , downloadInfoSha1 = Nothing
+                     , downloadInfoSha256 = Nothing
+                     })
         _ -> do
             ghcKey <- getGhcKey ghcBuild
             case Map.lookup ghcKey $ siGHCs si of
@@ -1140,8 +1153,8 @@ downloadAndInstallCompiler ghcBuild si wanted@WCGhc{} versionCheck mbindistURL =
     config <- view configL
     let installer =
             case configPlatform config of
-                Platform _ Cabal.Windows -> installGHCWindows (Just selectedVersion)
-                _ -> installGHCPosix (Just selectedVersion) downloadInfo
+                Platform _ Cabal.Windows -> installGHCWindows
+                _ -> installGHCPosix downloadInfo
     logInfo $
         "Preparing to install GHC" <>
         (case ghcVariant of
@@ -1245,6 +1258,9 @@ getOSKey platform =
         Platform X86_64                Cabal.Windows -> return "windows64"
         Platform Arm                   Cabal.Linux   -> return "linux-armv7"
         Platform AArch64               Cabal.Linux   -> return "linux-aarch64"
+        Platform Sparc                 Cabal.Linux   -> return "linux-sparc"
+        Platform AArch64               Cabal.OSX     -> return "macosx-aarch64"
+        Platform AArch64               Cabal.FreeBSD -> return "freebsd-aarch64"
         Platform arch os -> throwM $ UnsupportedSetupCombo os arch
 
 downloadOrUseLocal
@@ -1312,15 +1328,14 @@ data ArchiveType
     | SevenZ
 
 installGHCPosix :: HasConfig env
-                => Maybe Version
-                -> GHCDownloadInfo
+                => GHCDownloadInfo
                 -> SetupInfo
                 -> Path Abs File
                 -> ArchiveType
                 -> Path Abs Dir
                 -> Path Abs Dir
                 -> RIO env ()
-installGHCPosix mversion downloadInfo _ archiveFile archiveType tempDir destDir = do
+installGHCPosix downloadInfo _ archiveFile archiveType tempDir destDir = do
     platform <- view platformL
     menv0 <- view processContextL
     menv <- mkProcessContext (removeHaskellEnvVars (view envVarsL menv0))
@@ -1383,11 +1398,7 @@ installGHCPosix mversion downloadInfo _ archiveFile archiveType tempDir destDir
     logDebug $ "Unpacking " <> fromString (toFilePath archiveFile)
     runStep "unpacking" tempDir mempty tarTool [compOpt : "xf", toFilePath archiveFile]
 
-    dir <- case mversion of
-            Just version -> do
-               relDir <- parseRelDir $ "ghc-" ++ versionString version
-               return (tempDir </> relDir)
-            Nothing      -> expectSingleUnpackedDir archiveFile tempDir
+    dir <- expectSingleUnpackedDir archiveFile tempDir
 
     logSticky "Configuring GHC ..."
     runStep "configuring" dir
@@ -1433,27 +1444,24 @@ instance Alternative (CheckDependency env) where
             Right x' -> return $ Right x'
 
 installGHCWindows :: HasBuildConfig env
-                  => Maybe Version
-                  -> SetupInfo
+                  => SetupInfo
                   -> Path Abs File
                   -> ArchiveType
                   -> Path Abs Dir
                   -> Path Abs Dir
                   -> RIO env ()
-installGHCWindows mversion si archiveFile archiveType _tempDir destDir = do
-    tarComponent <- mapM (\v -> parseRelDir $ "ghc-" ++ versionString v) mversion
-    withUnpackedTarball7z "GHC" si archiveFile archiveType tarComponent destDir
+installGHCWindows si archiveFile archiveType _tempDir destDir = do
+    withUnpackedTarball7z "GHC" si archiveFile archiveType destDir
     logInfo $ "GHC installed to " <> fromString (toFilePath destDir)
 
 installMsys2Windows :: HasBuildConfig env
-                  => Text -- ^ OS Key
-                  -> SetupInfo
+                  => SetupInfo
                   -> Path Abs File
                   -> ArchiveType
                   -> Path Abs Dir
                   -> Path Abs Dir
                   -> RIO env ()
-installMsys2Windows osKey si archiveFile archiveType _tempDir destDir = do
+installMsys2Windows si archiveFile archiveType _tempDir destDir = do
     exists <- liftIO $ D.doesDirectoryExist $ toFilePath destDir
     when exists $ liftIO (D.removeDirectoryRecursive $ toFilePath destDir) `catchIO` \e -> do
         logError $
@@ -1461,8 +1469,7 @@ installMsys2Windows osKey si archiveFile archiveType _tempDir destDir = do
             fromString (toFilePath destDir)
         throwM e
 
-    msys <- parseRelDir $ "msys" ++ T.unpack (fromMaybe "32" $ T.stripPrefix "windows" osKey)
-    withUnpackedTarball7z "MSYS2" si archiveFile archiveType (Just msys) destDir
+    withUnpackedTarball7z "MSYS2" si archiveFile archiveType destDir
 
 
     -- I couldn't find this officially documented anywhere, but you need to run
@@ -1491,10 +1498,9 @@ withUnpackedTarball7z :: HasBuildConfig env
                       -> SetupInfo
                       -> Path Abs File -- ^ Path to archive file
                       -> ArchiveType
-                      -> Maybe (Path Rel Dir) -- ^ Name of directory expected in archive.  If Nothing, expects a single folder.
                       -> Path Abs Dir -- ^ Destination directory.
                       -> RIO env ()
-withUnpackedTarball7z name si archiveFile archiveType msrcDir destDir = do
+withUnpackedTarball7z name si archiveFile archiveType destDir = do
     suffix <-
         case archiveType of
             TarXz -> return ".xz"
@@ -1512,9 +1518,7 @@ withUnpackedTarball7z name si archiveFile archiveType msrcDir destDir = do
         liftIO $ ignoringAbsence (removeDirRecur destDir)
         run7z tmpDir archiveFile
         run7z tmpDir (tmpDir </> tarFile)
-        absSrcDir <- case msrcDir of
-            Just srcDir -> return $ tmpDir </> srcDir
-            Nothing -> expectSingleUnpackedDir archiveFile tmpDir
+        absSrcDir <- expectSingleUnpackedDir archiveFile tmpDir
         renameDir absSrcDir destDir
 
 expectSingleUnpackedDir :: (MonadIO m, MonadThrow m) => Path Abs File -> Path Abs Dir -> m (Path Abs Dir)
@@ -1785,14 +1789,92 @@ getUtf8EnvVars compilerVer =
 
 -- Binary Stack upgrades
 
-newtype StackReleaseInfo = StackReleaseInfo Value
+-- | Information on a binary release of Stack
+data StackReleaseInfo
+  = SRIGithub !Value
+  -- ^ Metadata downloaded from GitHub releases about available binaries.
+  | SRIHaskellStackOrg !HaskellStackOrg
+  -- ^ Information on the latest available binary for the current platforms.
 
-downloadStackReleaseInfo :: (MonadIO m, MonadThrow m)
-                         => Maybe String -- Github org
-                         -> Maybe String -- Github repo
-                         -> Maybe String -- ^ optional version
-                         -> m StackReleaseInfo
-downloadStackReleaseInfo morg mrepo mver = liftIO $ do
+data HaskellStackOrg = HaskellStackOrg
+  { hsoUrl :: !Text
+  , hsoVersion :: !Version
+  }
+  deriving Show
+
+downloadStackReleaseInfo
+  :: (HasPlatform env, HasLogFunc env)
+  => Maybe String -- Github org
+  -> Maybe String -- Github repo
+  -> Maybe String -- ^ optional version
+  -> RIO env StackReleaseInfo
+downloadStackReleaseInfo Nothing Nothing Nothing = do
+    platform <- view platformL
+    -- Fallback list of URLs to try for upgrading.
+    let urls0 =
+          case platform of
+            Platform X86_64 Cabal.Linux ->
+              [ "https://get.haskellstack.org/upgrade/linux-x86_64-static.tar.gz"
+              , "https://get.haskellstack.org/upgrade/linux-x86_64.tar.gz"
+              ]
+            Platform X86_64 Cabal.OSX ->
+              [ "https://get.haskellstack.org/upgrade/osx-x86_64.tar.gz"
+              ]
+            Platform X86_64 Cabal.Windows ->
+              [ "https://get.haskellstack.org/upgrade/windows-x86_64.tar.gz"
+              ]
+            _ -> []
+        -- Helper function: extract the version from a GitHub releases URL.
+    let extractVersion loc = do
+          version0 <-
+            case reverse $ splitOn "/" $ T.unpack loc of
+              _final:version0:_ -> Right version0
+              _ -> Left $ "Insufficient pieces in location: " ++ show loc
+          version1 <- maybe (Left "no leading v on version") Right $ stripPrefix "v" version0
+          maybe (Left $ "Invalid version: " ++ show version1) Right $ parseVersion version1
+
+        -- Try out different URLs. If we've exhausted all of them, fall back to GitHub.
+        loop [] = do
+          logDebug "Could not get binary from haskellstack.org, trying GitHub"
+          downloadStackReleaseInfoGithub Nothing Nothing Nothing
+
+        -- Try the next URL
+        loop (url:urls) = do
+          -- Make a HEAD request without any redirects
+          req <- setRequestMethod "HEAD" <$> parseRequest (T.unpack url)
+          res <- httpLbs req { redirectCount = 0 }
+
+          -- Look for a redirect. We're looking for a standard GitHub releases
+          -- URL where we can extract version information from.
+          case getResponseHeader "location" res of
+            [] -> logDebug "No location header found, continuing" *> loop urls
+            -- Exactly one location header.
+            [locBS] ->
+              case decodeUtf8' locBS of
+                Left e -> logDebug ("Invalid UTF8: " <> displayShow (locBS, e)) *> loop urls
+                Right loc ->
+                  case extractVersion loc of
+                    Left s -> logDebug ("No version found: " <> displayShow (url, loc, s)) *> loop (loc:urls)
+                    -- We found a valid URL, let's use it!
+                    Right version -> do
+                      let hso = HaskellStackOrg
+                                  { hsoUrl = loc
+                                  , hsoVersion = version
+                                  }
+                      logDebug $ "Downloading from haskellstack.org: " <> displayShow hso
+                      pure $ SRIHaskellStackOrg hso
+            locs -> logDebug ("Multiple location headers found: " <> displayShow locs) *> loop urls
+    loop urls0
+downloadStackReleaseInfo morg mrepo mver = downloadStackReleaseInfoGithub morg mrepo mver
+
+-- | Same as above, but always uses Github
+downloadStackReleaseInfoGithub
+  :: (MonadIO m, MonadThrow m)
+  => Maybe String -- Github org
+  -> Maybe String -- Github repo
+  -> Maybe String -- ^ optional version
+  -> m StackReleaseInfo
+downloadStackReleaseInfoGithub morg mrepo mver = liftIO $ do
     let org = fromMaybe "commercialhaskell" morg
         repo = fromMaybe "stack" mrepo
     let url = concat
@@ -1809,7 +1891,7 @@ downloadStackReleaseInfo morg mrepo mver = liftIO $ do
     res <- httpJSON $ setGithubHeaders req
     let code = getResponseStatusCode res
     if code >= 200 && code < 300
-        then return $ StackReleaseInfo $ getResponseBody res
+        then return $ SRIGithub $ getResponseBody res
         else throwString $ "Could not get release information for Stack from: " ++ url
 
 preferredPlatforms :: (MonadReader env m, HasPlatform env, MonadThrow m)
@@ -1904,7 +1986,7 @@ downloadStackExe platforms0 archiveInfo destDir checkPath testExe = do
       `catchAny` (logError . displayShow)
   where
 
-    findArchive (StackReleaseInfo val) pattern = do
+    findArchive (SRIGithub val) pattern = do
         Object top <- return val
         Array assets <- HashMap.lookup "assets" top
         getFirst $ fold $ fmap (First . findMatch pattern') assets
@@ -1918,6 +2000,7 @@ downloadStackExe platforms0 archiveInfo destDir checkPath testExe = do
             String url <- HashMap.lookup "browser_download_url" o
             Just url
         findMatch _ _ = Nothing
+    findArchive (SRIHaskellStackOrg hso) _ = pure $ hsoUrl hso
 
     handleTarball :: Path Abs File -> Bool -> T.Text -> IO ()
     handleTarball tmpFile isWindows url = do
@@ -1933,25 +2016,26 @@ downloadStackExe platforms0 archiveInfo destDir checkPath testExe = do
                     , T.unpack url
                     ]
                 loop (Tar.Fail e) = throwM e
-                loop (Tar.Next e es)
-                    | Tar.entryPath e == exeName =
-                        case Tar.entryContent e of
-                            Tar.NormalFile lbs _ -> do
-                              ensureDir destDir
-                              LBS.writeFile (toFilePath tmpFile) lbs
-                            _ -> error $ concat
-                                [ "Invalid file type for tar entry named "
-                                , exeName
-                                , " downloaded from "
-                                , T.unpack url
-                                ]
-                    | otherwise = loop es
+                loop (Tar.Next e es) =
+                    case FP.splitPath (Tar.entryPath e) of
+                        -- Ignore the first component, see: https://github.com/commercialhaskell/stack/issues/5288
+                        [_ignored, name] | name == exeName -> do
+                            case Tar.entryContent e of
+                                Tar.NormalFile lbs _ -> do
+                                  ensureDir destDir
+                                  LBS.writeFile (toFilePath tmpFile) lbs
+                                _ -> error $ concat
+                                    [ "Invalid file type for tar entry named "
+                                    , Tar.entryPath e
+                                    , " downloaded from "
+                                    , T.unpack url
+                                    ]
+                        _ -> loop es
             loop entries
       where
-        -- The takeBaseName drops the .gz, dropExtension drops the .tar
-        exeName =
-            let base = FP.dropExtension (FP.takeBaseName (T.unpack url)) FP.</> "stack"
-             in if isWindows then base FP.<.> "exe" else base
+        exeName
+          | isWindows = "stack.exe"
+          | otherwise = "stack"
 
 -- | Ensure that the Stack executable download is in the same location
 -- as the currently running executable. See:
@@ -2009,8 +2093,9 @@ performPathChecking newFile executablePath = do
         | otherwise -> throwM e
 
 getDownloadVersion :: StackReleaseInfo -> Maybe Version
-getDownloadVersion (StackReleaseInfo val) = do
+getDownloadVersion (SRIGithub val) = do
     Object o <- Just val
     String rawName <- HashMap.lookup "name" o
     -- drop the "v" at the beginning of the name
     parseVersion $ T.unpack (T.drop 1 rawName)
+getDownloadVersion (SRIHaskellStackOrg hso) = Just $ hsoVersion hso
