@@ -33,7 +33,6 @@ module Stack.Types.Config
   ,Config(..)
   ,HasConfig(..)
   ,askLatestSnapshotUrl
-  ,explicitSetupDeps
   ,configProjectRoot
   -- ** BuildConfig & HasBuildConfig
   ,BuildConfig(..)
@@ -321,6 +320,8 @@ data Config =
          -- ^ --extra-include-dirs arguments
          ,configExtraLibDirs        :: ![FilePath]
          -- ^ --extra-lib-dirs arguments
+         ,configCustomPreprocessorExts :: ![Text]
+         -- ^ List of custom preprocessors to complete the hard coded ones
          ,configConcurrentTests     :: !Bool
          -- ^ Run test suites concurrently
          ,configTemplateParams      :: !(Map Text Text)
@@ -342,8 +343,6 @@ data Config =
          -- ^ How PVP upper bounds should be added to packages
          ,configModifyCodePage      :: !Bool
          -- ^ Force the code page to UTF-8 on Windows
-         ,configExplicitSetupDeps   :: !(Map (Maybe PackageName) Bool)
-         -- ^ See 'explicitSetupDeps'. 'Nothing' provides the default value.
          ,configRebuildGhcOptions   :: !Bool
          -- ^ Rebuild on GHC options changes
          ,configApplyGhcOptions     :: !ApplyGhcOptions
@@ -801,6 +800,8 @@ data ConfigMonoid =
     -- ^ See: 'configExtraIncludeDirs'
     ,configMonoidExtraLibDirs        :: ![FilePath]
     -- ^ See: 'configExtraLibDirs'
+    ,configMonoidCustomPreprocessorExts :: ![Text]
+    -- ^ See: 'configCustomPreprocessorExts'
     , configMonoidOverrideGccPath    :: !(First (Path Abs File))
     -- ^ Allow users to override the path to gcc
     ,configMonoidOverrideHpack       :: !(First FilePath)
@@ -835,8 +836,6 @@ data ConfigMonoid =
     -- ^ See 'configPvpBounds'
     ,configMonoidModifyCodePage      :: !FirstTrue
     -- ^ See 'configModifyCodePage'
-    ,configMonoidExplicitSetupDeps   :: !(Map (Maybe PackageName) Bool)
-    -- ^ See 'configExplicitSetupDeps'
     ,configMonoidRebuildGhcOptions   :: !FirstFalse
     -- ^ See 'configMonoidRebuildGhcOptions'
     ,configMonoidApplyGhcOptions     :: !(First ApplyGhcOptions)
@@ -863,6 +862,8 @@ data ConfigMonoid =
     , configMonoidRecommendUpgrade   :: !FirstTrue
     -- ^ See 'configRecommendUpgrade'
     , configMonoidCasaRepoPrefix     :: !(First CasaRepoPrefix)
+    , configMonoidSnapshotLocation :: !(First Text)
+    -- ^ Custom location of LTS/Nightly snapshots
     , configMonoidStackDeveloperMode :: !(First Bool)
     -- ^ See 'configStackDeveloperMode'
     }
@@ -918,6 +919,7 @@ parseConfigMonoidObject rootDir obj = do
         obj ..:?  configMonoidExtraIncludeDirsName ..!= []
     configMonoidExtraLibDirs <- map (toFilePath rootDir FilePath.</>) <$>
         obj ..:?  configMonoidExtraLibDirsName ..!= []
+    configMonoidCustomPreprocessorExts <- obj ..:?  configMonoidCustomPreprocessorExtsName ..!= []
     configMonoidOverrideGccPath <- First <$> obj ..:? configMonoidOverrideGccPathName
     configMonoidOverrideHpack <- First <$> obj ..:? configMonoidOverrideHpackName
     configMonoidConcurrentTests <- First <$> obj ..:? configMonoidConcurrentTestsName
@@ -962,9 +964,6 @@ parseConfigMonoidObject rootDir obj = do
     configMonoidLocalProgramsBase <- First <$> obj ..:? configMonoidLocalProgramsBaseName
     configMonoidPvpBounds <- First <$> obj ..:? configMonoidPvpBoundsName
     configMonoidModifyCodePage <- FirstTrue <$> obj ..:? configMonoidModifyCodePageName
-    configMonoidExplicitSetupDeps <-
-        (obj ..:? configMonoidExplicitSetupDepsName ..!= mempty)
-        >>= fmap Map.fromList . mapM handleExplicitSetupDep . Map.toList
     configMonoidRebuildGhcOptions <- FirstFalse <$> obj ..:? configMonoidRebuildGhcOptionsName
     configMonoidApplyGhcOptions <- First <$> obj ..:? configMonoidApplyGhcOptionsName
     configMonoidAllowNewer <- First <$> obj ..:? configMonoidAllowNewerName
@@ -988,20 +987,11 @@ parseConfigMonoidObject rootDir obj = do
     configMonoidRecommendUpgrade <- FirstTrue <$> obj ..:? configMonoidRecommendUpgradeName
 
     configMonoidCasaRepoPrefix <- First <$> obj ..:? configMonoidCasaRepoPrefixName
+    configMonoidSnapshotLocation <- First <$> obj ..:? configMonoidSnapshotLocationName
 
     configMonoidStackDeveloperMode <- First <$> obj ..:? configMonoidStackDeveloperModeName
 
     return ConfigMonoid {..}
-  where
-    handleExplicitSetupDep :: (Monad m, MonadFail m) => (Text, Bool) -> m (Maybe PackageName, Bool)
-    handleExplicitSetupDep (name', b) = do
-        name <-
-            if name' == "*"
-                then return Nothing
-                else case parsePackageName $ T.unpack name' of
-                        Nothing -> fail $ "Invalid package name: " ++ show name'
-                        Just x -> return $ Just x
-        return (name, b)
 
 configMonoidWorkDirName :: Text
 configMonoidWorkDirName = "work-dir"
@@ -1063,6 +1053,9 @@ configMonoidExtraIncludeDirsName = "extra-include-dirs"
 configMonoidExtraLibDirsName :: Text
 configMonoidExtraLibDirsName = "extra-lib-dirs"
 
+configMonoidCustomPreprocessorExtsName  :: Text
+configMonoidCustomPreprocessorExtsName  = "custom-preprocessor-extensions"
+
 configMonoidOverrideGccPathName :: Text
 configMonoidOverrideGccPathName = "with-gcc"
 
@@ -1108,9 +1101,6 @@ configMonoidPvpBoundsName = "pvp-bounds"
 configMonoidModifyCodePageName :: Text
 configMonoidModifyCodePageName = "modify-code-page"
 
-configMonoidExplicitSetupDepsName :: Text
-configMonoidExplicitSetupDepsName = "explicit-setup-deps"
-
 configMonoidRebuildGhcOptionsName :: Text
 configMonoidRebuildGhcOptionsName = "rebuild-ghc-options"
 
@@ -1156,6 +1146,9 @@ configMonoidRecommendUpgradeName = "recommend-stack-upgrade"
 configMonoidCasaRepoPrefixName :: Text
 configMonoidCasaRepoPrefixName = "casa-repo-prefix"
 
+configMonoidSnapshotLocationName :: Text
+configMonoidSnapshotLocationName = "snapshot-location-base"
+
 configMonoidStackDeveloperModeName :: Text
 configMonoidStackDeveloperModeName = "stack-developer-mode"
 
@@ -1219,11 +1212,14 @@ instance Show ConfigException where
         , ") is outside the required\n"
         ,"version range specified in stack.yaml ("
         , T.unpack (versionRangeText requiredRange)
-        , ")." ]
+        , ").\n"
+        , "You can upgrade stack by running:\n\n"
+        , "stack upgrade"
+        ]
     show (NoMatchingSnapshot names) = concat
         [ "None of the following snapshots provides a compiler matching "
         , "your package(s):\n"
-        , unlines $ map (\name -> "    - " <> T.unpack (renderSnapName name))
+        , unlines $ map (\name -> "    - " <> show name)
                         (NonEmpty.toList names)
         , resolveOptions
         ]
@@ -1770,16 +1766,6 @@ instance ToJSON PvpBounds where
 instance FromJSON PvpBounds where
   parseJSON = withText "PvpBounds" (either fail return . parsePvpBounds)
 
--- | Provide an explicit list of package dependencies when running a custom Setup.hs
-explicitSetupDeps :: (MonadReader env m, HasConfig env) => PackageName -> m Bool
-explicitSetupDeps name = do
-    m <- view $ configL.to configExplicitSetupDeps
-    return $
-      Map.findWithDefault
-        (Map.findWithDefault False Nothing m)
-        (Just name)
-        m
-
 -- | Data passed into Docker container for the Docker entrypoint's use
 newtype DockerEntrypoint = DockerEntrypoint
     { deUser :: Maybe DockerUser
