@@ -12,8 +12,12 @@ module Stack.Upload
     , uploadRevision
       -- * Credentials
     , HackageCreds
-    , loadCreds
+    , HackageAuth(..)
+    , HackageKey(..)
+    , loadAuth
     , writeFilePrivate
+      -- * Internal
+    , maybeGetHackageKey
     ) where
 
 import           Stack.Prelude
@@ -27,7 +31,13 @@ import qualified Data.ByteString.Char8                 as S
 import qualified Data.ByteString.Lazy                  as L
 import qualified Data.Conduit.Binary                   as CB
 import qualified Data.Text                             as T
-import           Network.HTTP.StackClient              (Request, RequestBody(RequestBodyLBS), Response, withResponse, httpNoBody, getGlobalManager, getResponseStatusCode,
+import           Network.HTTP.StackClient              (Request,
+                                                        RequestBody(RequestBodyLBS),
+                                                        Response,
+                                                        withResponse,
+                                                        httpNoBody,
+                                                        getGlobalManager,
+                                                        getResponseStatusCode,
                                                         getResponseBody,
                                                         setRequestHeader,
                                                         parseRequest,
@@ -35,14 +45,18 @@ import           Network.HTTP.StackClient              (Request, RequestBody(Req
                                                         partBS, partLBS,
                                                         applyDigestAuth,
                                                         displayDigestAuthException)
+import           Stack.Options.UploadParser
 import           Stack.Types.Config
 import           System.Directory                      (createDirectoryIfMissing,
                                                         removeFile, renameFile)
 import           System.Environment                    (lookupEnv)
 import           System.FilePath                       ((</>), takeFileName, takeDirectory)
-import           System.IO                             (putStrLn, putStr, print) -- TODO remove putStrLn, use logInfo
 import           System.PosixCompat.Files              (setFileMode)
 
+
+newtype HackageKey = HackageKey Text
+    deriving (Eq, Show)
+
 -- | Username and password to log into Hackage.
 --
 -- Since 0.1.0.0
@@ -51,7 +65,11 @@ data HackageCreds = HackageCreds
     , hcPassword :: !Text
     , hcCredsFile :: !FilePath
     }
-    deriving Show
+    deriving (Eq, Show)
+
+data HackageAuth = HAKey HackageKey
+                 | HACreds HackageCreds
+    deriving (Eq, Show)
 
 instance ToJSON HackageCreds where
     toJSON (HackageCreds u p _) = object
@@ -66,14 +84,26 @@ instance FromJSON (FilePath -> HackageCreds) where
 withEnvVariable :: Text -> IO Text -> IO Text
 withEnvVariable varName fromPrompt = lookupEnv (T.unpack varName) >>= maybe fromPrompt (pure . T.pack)
 
+maybeGetHackageKey :: RIO m (Maybe HackageKey)
+maybeGetHackageKey = liftIO $ fmap (HackageKey . T.pack) <$> lookupEnv "HACKAGE_KEY"
+
+loadAuth :: HasLogFunc m => Config -> RIO m HackageAuth
+loadAuth config = do
+  maybeHackageKey <- maybeGetHackageKey
+  case maybeHackageKey of
+    Just key -> do
+      logInfo "HACKAGE_KEY found in env, using that for credentials."
+      return $ HAKey key
+    Nothing -> HACreds <$> loadUserAndPassword config
+
 -- | Load Hackage credentials, either from a save file or the command
 -- line.
 --
 -- Since 0.1.0.0
-loadCreds :: Config -> IO HackageCreds
-loadCreds config = do
-  fp <- credsFile config
-  elbs <- tryIO $ L.readFile fp
+loadUserAndPassword :: HasLogFunc m => Config -> RIO m HackageCreds
+loadUserAndPassword config = do
+  fp <- liftIO $ credsFile config
+  elbs <- liftIO $ tryIO $ L.readFile fp
   case either (const Nothing) Just elbs >>= \lbs -> (lbs, ) <$> decode' lbs of
     Nothing -> fromPrompt fp
     Just (lbs, mkCreds) -> do
@@ -82,14 +112,15 @@ loadCreds config = do
       writeFilePrivate fp $ lazyByteString lbs
 
       unless (configSaveHackageCreds config) $ do
-        putStrLn "WARNING: You've set save-hackage-creds to false"
-        putStrLn "However, credentials were found at:"
-        putStrLn $ "  " ++ fp
+        logWarn "WARNING: You've set save-hackage-creds to false"
+        logWarn "However, credentials were found at:"
+        logWarn $ "  " <> fromString fp
       return $ mkCreds fp
   where
+    fromPrompt :: HasLogFunc m => FilePath -> RIO m HackageCreds
     fromPrompt fp = do
-      username <- withEnvVariable "HACKAGE_USERNAME" (prompt "Hackage username: ")
-      password <- withEnvVariable "HACKAGE_PASSWORD" (promptPassword "Hackage password: ")
+      username <- liftIO $ withEnvVariable "HACKAGE_USERNAME" (prompt "Hackage username: ")
+      password <- liftIO $ withEnvVariable "HACKAGE_PASSWORD" (promptPassword "Hackage password: ")
       let hc = HackageCreds
             { hcUsername = username
             , hcPassword = password
@@ -99,10 +130,10 @@ loadCreds config = do
       when (configSaveHackageCreds config) $ do
         shouldSave <- promptBool $ T.pack $
           "Save hackage credentials to file at " ++ fp ++ " [y/n]? "
-        putStrLn "NOTE: Avoid this prompt in the future by using: save-hackage-creds: false"
+        logInfo "NOTE: Avoid this prompt in the future by using: save-hackage-creds: false"
         when shouldSave $ do
           writeFilePrivate fp $ fromEncoding $ toEncoding hc
-          putStrLn "Saved!"
+          logInfo "Saved!"
           hFlush stdout
 
       return hc
@@ -135,20 +166,30 @@ credsFile config = do
     createDirectoryIfMissing True dir
     return $ dir </> "credentials.json"
 
-applyCreds :: HackageCreds -> Request -> IO Request
+addAPIKey :: HackageKey -> Request -> Request
+addAPIKey (HackageKey key) req =
+  setRequestHeader "Authorization" [fromString $ "X-ApiKey" ++ " " ++ T.unpack key] req
+
+applyAuth :: HasLogFunc m => HackageAuth -> Request -> RIO m Request
+applyAuth haAuth req0 = do
+    case haAuth of
+        HAKey key -> return (addAPIKey key req0)
+        HACreds creds -> applyCreds creds req0
+
+applyCreds :: HasLogFunc m => HackageCreds -> Request -> RIO m Request
 applyCreds creds req0 = do
-  manager <- getGlobalManager
-  ereq <- applyDigestAuth
+  manager <- liftIO getGlobalManager
+  ereq <- liftIO $ applyDigestAuth
     (encodeUtf8 $ hcUsername creds)
     (encodeUtf8 $ hcPassword creds)
     req0
     manager
   case ereq of
       Left e -> do
-          putStrLn "WARNING: No HTTP digest prompt found, this will probably fail"
+          logWarn "WARNING: No HTTP digest prompt found, this will probably fail"
           case fromException e of
-              Just e' -> putStrLn $ displayDigestAuthException e'
-              Nothing -> print e
+              Just e' -> logWarn $ fromString $ displayDigestAuthException e'
+              Nothing -> logWarn $ fromString $ displayException e
           return req0
       Right req -> return req
 
@@ -156,40 +197,52 @@ applyCreds creds req0 = do
 -- sending a file like 'upload', this sends a lazy bytestring.
 --
 -- Since 0.1.2.1
-uploadBytes :: String -- ^ Hackage base URL
-            -> HackageCreds
+uploadBytes :: HasLogFunc m
+            => String -- ^ Hackage base URL
+            -> HackageAuth
             -> String -- ^ tar file name
+            -> UploadVariant
             -> L.ByteString -- ^ tar file contents
-            -> IO ()
-uploadBytes baseUrl creds tarName bytes = do
+            -> RIO m ()
+uploadBytes baseUrl auth tarName uploadVariant bytes = do
     let req1 = setRequestHeader "Accept" ["text/plain"]
-               (fromString $ baseUrl <> "packages/")
+               (fromString $ baseUrl
+                          <> "packages/"
+                          <> case uploadVariant of
+                               Publishing -> ""
+                               Candidate -> "candidates/"
+               )
         formData = [partFileRequestBody "package" tarName (RequestBodyLBS bytes)]
-    req2 <- formDataBody formData req1
-    req3 <- applyCreds creds req2
-    putStr $ "Uploading " ++ tarName ++ "... "
+    req2 <- liftIO $ formDataBody formData req1
+    req3 <- applyAuth auth req2
+    logInfo $ "Uploading " <> fromString tarName <> "... "
     hFlush stdout
-    withResponse req3 $ \res ->
+    withRunInIO $ \runInIO -> withResponse req3 (runInIO . inner)
+ where
+    inner :: HasLogFunc m => Response (ConduitM () S.ByteString IO ()) -> RIO m ()
+    inner res =
         case getResponseStatusCode res of
-            200 -> putStrLn "done!"
+            200 -> logInfo "done!"
             401 -> do
-                putStrLn "authentication failure"
-                handleIO (const $ return ()) (removeFile (hcCredsFile creds))
+                logError "authentication failure"
+                case auth of
+                  HACreds creds -> handleIO (const $ return ()) (liftIO $ removeFile (hcCredsFile creds))
+                  _ -> pure ()
                 throwString "Authentication failure uploading to server"
             403 -> do
-                putStrLn "forbidden upload"
-                putStrLn "Usually means: you've already uploaded this package/version combination"
-                putStrLn "Ignoring error and continuing, full message from Hackage below:\n"
-                printBody res
+                logError "forbidden upload"
+                logError "Usually means: you've already uploaded this package/version combination"
+                logError "Ignoring error and continuing, full message from Hackage below:\n"
+                liftIO $ printBody res
             503 -> do
-                putStrLn "service unavailable"
-                putStrLn "This error some times gets sent even though the upload succeeded"
-                putStrLn "Check on Hackage to see if your pacakge is present"
-                printBody res
+                logError "service unavailable"
+                logError "This error some times gets sent even though the upload succeeded"
+                logError "Check on Hackage to see if your pacakge is present"
+                liftIO $ printBody res
             code -> do
-                putStrLn $ "unhandled status code: " ++ show code
-                printBody res
-                throwString $ "Upload failed on " ++ tarName
+                logError $ "unhandled status code: " <> fromString (show code)
+                liftIO $ printBody res
+                throwString $ "Upload failed on " <> fromString tarName
 
 printBody :: Response (ConduitM () S.ByteString IO ()) -> IO ()
 printBody res = runConduit $ getResponseBody res .| CB.sinkHandle stdout
@@ -197,18 +250,22 @@ printBody res = runConduit $ getResponseBody res .| CB.sinkHandle stdout
 -- | Upload a single tarball with the given @Uploader@.
 --
 -- Since 0.1.0.0
-upload :: String -- ^ Hackage base URL
-       -> HackageCreds
+upload :: HasLogFunc m
+       => String -- ^ Hackage base URL
+       -> HackageAuth
        -> FilePath
-       -> IO ()
-upload baseUrl creds fp = uploadBytes baseUrl creds (takeFileName fp) =<< L.readFile fp
+       -> UploadVariant
+       -> RIO m ()
+upload baseUrl auth fp uploadVariant =
+  uploadBytes baseUrl auth (takeFileName fp) uploadVariant =<< liftIO (L.readFile fp)
 
-uploadRevision :: String -- ^ Hackage base URL
-               -> HackageCreds
+uploadRevision :: HasLogFunc m
+               => String -- ^ Hackage base URL
+               -> HackageAuth
                -> PackageIdentifier
                -> L.ByteString
-               -> IO ()
-uploadRevision baseUrl creds ident@(PackageIdentifier name _) cabalFile = do
+               -> RIO m ()
+uploadRevision baseUrl auth ident@(PackageIdentifier name _) cabalFile = do
   req0 <- parseRequest $ concat
     [ baseUrl
     , "package/"
@@ -222,5 +279,5 @@ uploadRevision baseUrl creds ident@(PackageIdentifier name _) cabalFile = do
     , partBS "publish" "on"
     ]
     req0
-  req2 <- applyCreds creds req1
+  req2 <- applyAuth auth req1
   void $ httpNoBody req2
