@@ -50,6 +50,7 @@ import qualified Stack.Nix as Nix
 import           Stack.FileWatch
 import           Stack.Ghci
 import           Stack.Hoogle
+import           Stack.List
 import           Stack.Ls
 import qualified Stack.IDE as IDE
 import           Stack.Init
@@ -67,6 +68,7 @@ import           Stack.Options.NewParser
 import           Stack.Options.NixParser
 import           Stack.Options.ScriptParser
 import           Stack.Options.SDistParser
+import           Stack.Options.UploadParser
 import           Stack.Options.Utils
 import qualified Stack.Path
 import           Stack.Runners
@@ -262,7 +264,7 @@ commandLineHandler currentDir progName isInterpreter = complicatedOptions
             "upload"
             "Upload a package to Hackage"
             uploadCmd
-            sdistOptsParser
+            uploadOptsParser
         addCommand'
             "sdist"
             "Create source distribution tarballs"
@@ -277,11 +279,12 @@ commandLineHandler currentDir progName isInterpreter = complicatedOptions
                     execCmd
                     (execOptsParser $ Just ExecGhc)
         addCommand' "hoogle"
-                    ("Run hoogle, the Haskell API search engine. Use 'stack exec' syntax " ++
-                     "to pass Hoogle arguments, e.g. stack hoogle -- --count=20 or " ++
-                     "stack hoogle -- server --local")
+                    ("Run hoogle, the Haskell API search engine. Use the '-- ARGUMENT(S)' syntax " ++
+                     "to pass Hoogle arguments, e.g. stack hoogle -- --count=20, or " ++
+                     "stack hoogle -- server --local.")
                     hoogleCmd
-                    ((,,,) <$> many (strArgument (metavar "ARG"))
+                    ((,,,) <$> many (strArgument
+                                 (metavar "-- ARGUMENT(S) (e.g. stack hoogle -- server --local)"))
                           <*> boolFlags
                                   True
                                   "setup"
@@ -297,7 +300,7 @@ commandLineHandler currentDir progName isInterpreter = complicatedOptions
 
       -- These are the only commands allowed in interpreter mode as well
       addCommand' "exec"
-                  "Execute a command"
+                  "Execute a command. If the command is absent, the first of any arguments is taken as the command."
                   execCmd
                   (execOptsParser Nothing)
       addCommand' "run"
@@ -348,6 +351,10 @@ commandLineHandler currentDir progName isInterpreter = complicatedOptions
                     "Query general build information (experimental)"
                     queryCmd
                     (many $ strArgument $ metavar "SELECTOR...")
+        addCommand' "list"
+                    "List package id's in snapshot (experimental)"
+                    listCmd
+                    (many $ strArgument $ metavar "PACKAGE")
         addSubCommands'
             "ide"
             "IDE-specific commands"
@@ -547,7 +554,7 @@ cleanCmd = withConfig NoReexec . withBuildConfig . clean
 -- | Helper for build and install commands
 buildCmd :: BuildOptsCLI -> RIO Runner ()
 buildCmd opts = do
-  when (any (("-prof" `elem`) . either (const []) id . parseArgs Escaping) (boptsCLIGhcOptions opts)) $ do
+  when (any (("-prof" `elem`) . fromRight [] . parseArgs Escaping) (boptsCLIGhcOptions opts)) $ do
     logError "Error: When building with stack, you should not use the -prof GHC option"
     logError "Instead, please use --library-profiling and --executable-profiling"
     logError "See: https://github.com/commercialhaskell/stack/issues/1015"
@@ -612,20 +619,21 @@ upgradeCmd upgradeOpts' = do
         upgradeOpts'
 
 -- | Upload to Hackage
-uploadCmd :: SDistOpts -> RIO Runner ()
-uploadCmd (SDistOpts [] _ _ _ _) = do
+uploadCmd :: UploadOpts -> RIO Runner ()
+uploadCmd (UploadOpts (SDistOpts [] _ _ _ _) _) = do
     prettyErrorL
         [ flow "To upload the current package, please run"
         , PP.style Shell "stack upload ."
         , flow "(with the period at the end)"
         ]
     liftIO exitFailure
-uploadCmd sdistOpts = do
+uploadCmd uploadOpts = do
     let partitionM _ [] = return ([], [])
         partitionM f (x:xs) = do
             r <- f x
             (as, bs) <- partitionM f xs
             return $ if r then (x:as, bs) else (as, x:bs)
+        sdistOpts = uoptsSDistOpts uploadOpts
     (files, nonFiles) <- liftIO $ partitionM D.doesFileExist (sdoptsDirsToWorkWith sdistOpts)
     (dirs, invalid) <- liftIO $ partitionM D.doesDirectoryExist nonFiles
     withConfig YesReexec $ withDefaultEnvConfig $ do
@@ -646,24 +654,20 @@ uploadCmd sdistOpts = do
             exitFailure
         config <- view configL
         let hackageUrl = T.unpack $ configHackageBaseUrl config
-        getCreds <- liftIO $ memoizeRef $ Upload.loadCreds config
+            uploadVariant = uoptsUploadVariant uploadOpts
+        getCreds <- memoizeRef $ Upload.loadAuth config
         mapM_ (resolveFile' >=> checkSDistTarball sdistOpts) files
-        forM_
-            files
-            (\file ->
-                  do tarFile <- resolveFile' file
-                     liftIO $ do
-                       creds <- runMemoized getCreds
-                       Upload.upload hackageUrl creds (toFilePath tarFile))
-        unless (null dirs) $
-            forM_ dirs $ \dir -> do
-                pkgDir <- resolveDir' dir
-                (tarName, tarBytes, mcabalRevision) <- getSDistTarball (sdoptsPvpBounds sdistOpts) pkgDir
-                checkSDistTarball' sdistOpts tarName tarBytes
-                liftIO $ do
-                  creds <- runMemoized getCreds
-                  Upload.uploadBytes hackageUrl creds tarName tarBytes
-                  forM_ mcabalRevision $ uncurry $ Upload.uploadRevision hackageUrl creds
+        forM_ files $ \file -> do
+            tarFile <- resolveFile' file
+            creds <- runMemoized getCreds
+            Upload.upload hackageUrl creds (toFilePath tarFile) uploadVariant
+        forM_ dirs $ \dir -> do
+            pkgDir <- resolveDir' dir
+            (tarName, tarBytes, mcabalRevision) <- getSDistTarball (sdoptsPvpBounds sdistOpts) pkgDir
+            checkSDistTarball' sdistOpts tarName tarBytes
+            creds <- runMemoized getCreds
+            Upload.uploadBytes hackageUrl creds tarName uploadVariant tarBytes
+            forM_ mcabalRevision $ uncurry $ Upload.uploadRevision hackageUrl creds
 
 sdistCmd :: SDistOpts -> RIO Runner ()
 sdistCmd sdistOpts =
@@ -849,7 +853,17 @@ templatesCmd () = withConfig NoReexec templatesHelp
 queryCmd :: [String] -> RIO Runner ()
 queryCmd selectors = withConfig YesReexec $ withDefaultEnvConfig $ queryBuildInfo $ map T.pack selectors
 
--- | Generate a combined HPC report
+-- | List packages
+listCmd :: [String] -> RIO Runner ()
+listCmd names = withConfig NoReexec $ do
+    mresolver <- view $ globalOptsL.to globalResolver
+    mSnapshot <- forM mresolver $ \resolver -> do
+      concrete <- makeConcreteResolver resolver
+      loc <- completeSnapshotLocation concrete
+      loadSnapshot loc
+    listPackages mSnapshot names
+
+-- | generate a combined HPC report
 hpcReportCmd :: HpcReportOpts -> RIO Runner ()
 hpcReportCmd hropts = do
     let (tixFiles, targetNames) = partition (".tix" `T.isSuffixOf`) (hroptsInputs hropts)
