@@ -9,6 +9,7 @@ import Stack.Upload
 import Test.Hspec
 import System.Permissions (osIsWindows)
 import System.PosixCompat.Files (getFileStatus, fileMode)
+import System.Environment (setEnv, unsetEnv)
 import Data.Bits ((.&.))
 
 spec :: Spec
@@ -26,3 +27,14 @@ spec = do
     unless osIsWindows $ do
       status <- getFileStatus fp
       (fileMode status .&. 0o777) `shouldBe` 0o600
+
+  it "finds a HACKAGE_KEY env variable" $ do
+    runRIO () maybeGetHackageKey `shouldReturn` Nothing
+
+    withEnv "HACKAGE_KEY" "api_key"
+      $ runRIO () maybeGetHackageKey `shouldReturn` Just (HackageKey "api_key")
+
+withEnv :: String -> String -> IO a -> IO a
+withEnv k v f = do
+  setEnv k v
+  f `finally` unsetEnv k
